@@ -5,8 +5,9 @@ A script to provision a Compute Requirement.
 """
 
 from dataclasses import dataclass
+from json import loads as json_loads
 from math import ceil, floor
-from typing import List, Optional
+from typing import List
 
 import requests
 from yellowdog_client.model import (
@@ -14,14 +15,10 @@ from yellowdog_client.model import (
     ComputeRequirementTemplateUsage,
 )
 
-from yd_commands.config import (
-    ARGS_PARSER,
-    WP_VARIABLES_PREFIX,
-    ConfigWorkerPool,
-    generate_id,
-    link_entity,
-    load_config_worker_pool,
-)
+from yd_commands.config_types import WP_VARIABLES_PREFIX, ConfigWorkerPool
+from yd_commands.follow_utils import follow_events, follow_ids
+from yd_commands.id_utils import YDIDType
+from yd_commands.load_config import load_config_worker_pool
 from yd_commands.printing import (
     print_compute_template_test_result,
     print_error,
@@ -29,11 +26,12 @@ from yd_commands.printing import (
     print_yd_object,
 )
 from yd_commands.provision_utils import get_user_data_property
+from yd_commands.utils import add_batch_number_postfix, generate_id, link_entity
 from yd_commands.variables import (
     load_json_file_with_variable_substitutions,
     load_jsonnet_file_with_variable_substitutions,
 )
-from yd_commands.wrapper import CLIENT, CONFIG_COMMON, main_wrapper
+from yd_commands.wrapper import ARGS_PARSER, CLIENT, CONFIG_COMMON, main_wrapper
 
 
 # Specifies the number of instances in a Compute Requirement batch
@@ -43,6 +41,7 @@ class CRBatch:
 
 
 CONFIG_WP: ConfigWorkerPool = load_config_worker_pool()
+GENERATED_ID = generate_id("cr" + "_" + CONFIG_COMMON.name_tag)
 
 
 @main_wrapper
@@ -59,16 +58,9 @@ def main():
     if cr_json_file is None:  # Finally, try 'computeRequirementData'
         cr_json_file = CONFIG_WP.compute_requirement_data_file
 
-    # Use the variable prefix if this is a Worker Pool file
     if cr_json_file is not None:
-        if (
-            ARGS_PARSER.worker_pool_file or CONFIG_WP.worker_pool_data_file
-        ) and not ARGS_PARSER.compute_requirement:
-            prefix = WP_VARIABLES_PREFIX
-        else:
-            prefix = ""
         print_log(f"Loading Compute Requirement data from: '{cr_json_file}'")
-        create_compute_requirement_from_json(cr_json_file, prefix)
+        create_compute_requirement_from_json(cr_json_file, WP_VARIABLES_PREFIX)
         return
 
     if CONFIG_WP.template_id is None:
@@ -89,9 +81,12 @@ def main():
     if num_batches > 1 and not ARGS_PARSER.report:
         print_log(f"Batching into {num_batches} Compute Requirements")
 
+    compute_requirement_ids: List[str] = []
     for batch_number in range(num_batches):
-        id = generate_cr_batch_name(
-            name=CONFIG_WP.name, batch_number=batch_number, num_batches=num_batches
+        id = add_batch_number_postfix(
+            name=CONFIG_WP.name if CONFIG_WP.name is not None else GENERATED_ID,
+            batch_number=batch_number,
+            num_batches=num_batches,
         )
         if not (ARGS_PARSER.dry_run or ARGS_PARSER.report):
             if num_batches > 1:
@@ -125,8 +120,16 @@ def main():
                     )
                     print_compute_template_test_result(test_result)
                 except requests.HTTPError as http_error:
+                    if http_error.response.status_code == 404:
+                        raise Exception(
+                            json_loads(http_error.response.text).get(
+                                "message", "Template ID not found"
+                            )
+                        )
                     if "No sources" in http_error.response.text:
                         print_log("No Compute Sources match the Template's constraints")
+                    else:
+                        raise http_error
                 return
 
             if not ARGS_PARSER.dry_run:
@@ -135,19 +138,28 @@ def main():
                         compute_requirement_template_usage
                     )
                 )
+                compute_requirement_ids.append(compute_requirement.id)
                 if ARGS_PARSER.quiet:
                     print(compute_requirement.id)
                 print_log(
                     f"Provisioned {link_entity(CONFIG_COMMON.url, compute_requirement)}"
                 )
+                print_log(f"YellowDog ID is '{compute_requirement.id}'")
+
             else:
                 print_log("Dry-run: Printing JSON Compute Requirement specification")
                 print_yd_object(compute_requirement_template_usage)
                 print_log("Dry-run: Complete")
 
         except Exception as e:
-            print_error(f"Unable to provision Compute Requirement")
-            raise Exception(e)
+            raise Exception(
+                "Unable to"
+                f" {'report on' if ARGS_PARSER.report else 'provision'} Compute"
+                f" Requirement: {e}"
+            )
+
+    if ARGS_PARSER.follow:
+        follow_ids(compute_requirement_ids)
 
 
 def _allocate_nodes_to_batches(
@@ -183,26 +195,9 @@ def _allocate_nodes_to_batches(
     return batches
 
 
-def generate_cr_batch_name(
-    name: Optional[str], batch_number: int, num_batches: int
-) -> str:
-    """
-    Generate the name of a Worker Pool
-    """
-
-    # Standard automatic name generation
-    if name is None:
-        return generate_id("cr" + "_" + CONFIG_COMMON.name_tag)
-
-    # Use supplied name, with counter if multiple batches
-    if num_batches > 1:
-        name += "_" + str(batch_number + 1).zfill(len(str(num_batches)))
-    return name
-
-
 def create_compute_requirement_from_json(cr_json_file: str, prefix: str = "") -> None:
     """
-    Directly create the Compute Requirement using the YellowDog REST API
+    Directly create the Compute Requirement using the YellowDog REST API.
     """
 
     if ARGS_PARSER.report:
@@ -231,11 +226,7 @@ def create_compute_requirement_from_json(cr_json_file: str, prefix: str = "") ->
             # Generate a default name
             (
                 "requirementName",
-                (
-                    CONFIG_WP.name
-                    if CONFIG_WP.name is not None
-                    else generate_id("cr" + "_" + CONFIG_COMMON.name_tag)
-                ),
+                (CONFIG_WP.name if CONFIG_WP.name is not None else GENERATED_ID),
             ),
             ("requirementNamespace", CONFIG_COMMON.namespace),
             ("requirementTag", CONFIG_COMMON.name_tag),
@@ -272,6 +263,9 @@ def create_compute_requirement_from_json(cr_json_file: str, prefix: str = "") ->
         print_log(f"Provisioned Compute Requirement '{name}' ({id})")
         if ARGS_PARSER.quiet:
             print(id)
+        if ARGS_PARSER.follow:
+            print_log("Following Compute Requirement event stream")
+            follow_events(id, YDIDType.COMPUTE_REQ)
     else:
         print_error(f"Failed to provision Compute Requirement '{name}'")
         raise Exception(f"{response.text}")
